//-----------------------------------------------------------------------------
// Torque 2D. 
// Copyright (C) GarageGames.com, Inc.
//-----------------------------------------------------------------------------



// --------------------------------------------------------------------
// Bit-Shifter Helper.
// --------------------------------------------------------------------
function BIT(%bitIndex)
{
	return 1<<%bitIndex;
}


//-----------------------------------------------------------------------------
// Add Mod-Token to front of mods list.
//-----------------------------------------------------------------------------
function pushFront(%list, %token, %delim)
{
	// Is the list empty?
	if (%list !$= "")
		// No, so add new token.
		return %token @ %delim @ %list;
		
	// Yes, so just use new token.
	return %token;
}

//-----------------------------------------------------------------------------
// Add Mod-Token to back of mods list.
//-----------------------------------------------------------------------------
function pushBack(%list, %token, %delim)
{
	// Is the list empty?
	if (%list !$= "")
		// No, so add new token.
		return %list @ %delim @ %token;
		
	// Yes, so just use new token.
	return %token;
}


//-----------------------------------------------------------------------------
// Remove Mod-Token from the mods list.
//-----------------------------------------------------------------------------
function popFront(%list, %delim)
{
	// Next Token.
	return nextToken(%list, unused, %delim);
}


//-----------------------------------------------------------------------------
// The displayHelp, onStart, onExit and parseArgs function are overriden
// by mod packages to get hooked into initialization and cleanup. 
//-----------------------------------------------------------------------------
function onStart()
{
   // Default startup function
}


//-----------------------------------------------------------------------------
// Exit Engine.
//-----------------------------------------------------------------------------
function onExit()
{
   // OnExit is called directly from C++ code, whereas onStart is
   // invoked at the end of this file.
}


//-----------------------------------------------------------------------------
// Parse Arguments.
//-----------------------------------------------------------------------------
function parseArgs()
{
   // Here for mod override, the arguments have already
   // been parsed.
}   


//-----------------------------------------------------------------------------
// Help Package.
//-----------------------------------------------------------------------------
package Help
{
   function onExit()
   {
      // Override onExit when displaying help
   }
};


//-----------------------------------------------------------------------------
// Default Help Display.
//-----------------------------------------------------------------------------
function displayHelp()
{

}


//--------------------------------------------------------------------------
// Execute MOD Directory.
//--------------------------------------------------------------------------
function loadDir( %dir )
{
	// Set Mod Paths.
	setModPaths( pushback($userMods, %dir, ";") );
	
	// Execute Boot-strap file.
	exec( %dir @ "/main.cs" );
}


//--------------------------------------------------------------------------
// Execute startup scripts for each mod, starting at base and working up.
//--------------------------------------------------------------------------
function loadMods( %modPath )
{
	// Get MOD Path.
	%modPath = nextToken(%modPath, token, ";");
	
	// Did we get a MOD token?
	if (%modPath !$= "")
		// Yes, so recursively load it.
		loadMods(%modPath);

	// Execute MOD Boot-strap file.
	if( exec(%token @ "/main.cs") != true )
	{
		// Problem.
		error("Error: Unable to find specified mod: " @ %token );
		// Decrease MOD Count.
		$modcount--;
	}
}





//------------------------------------------------------------------------------
// PROGRAM MAIN LINE
//------------------------------------------------------------------------------

$modcount = 1;
$defaultGame = "PAS";
$userMods = $defaultGame;
$displayHelp = false;


//------------------------------------------------------------------------------
// Process command line arguments
//------------------------------------------------------------------------------
for ($i = 1; $i < $Game::argc ; $i++)
{
	// Set Current Argument.
	$arg = $Game::argv[$i];
	// Set Next Argument.
	$nextArg = $Game::argv[$i+1];
	// Check if we've got another argument.
	$hasNextArg = $Game::argc - $i > 1;
	// Log-Mode Off by default.
	$logModeSpecified = false;
}


$userMods = $defaultGame;
$modcount = 1;

//--------------------------------------------------------------------------
// Default to a new logfile each session.
//--------------------------------------------------------------------------
if ( !$logModeSpecified )
{
   setLogMode(6);
}

//--------------------------------------------------------------------------
// Set the mod path which dictates which directories will be visible
// to the scripts and the resource engine.
//--------------------------------------------------------------------------
setModPaths( $userMods );

//--------------------------------------------------------------------------
// Get the first mod on the list, which will be the last to be applied... this
// does not modify the list.
//--------------------------------------------------------------------------
nextToken( $userMods, currentMod, ";" );

//--------------------------------------------------------------------------
// Start the MOD loading process...
//--------------------------------------------------------------------------
loadMods( $userMods );
echo( "" );

//--------------------------------------------------------------------------
// Parse the command line arguments.
//--------------------------------------------------------------------------
parseArgs();


//--------------------------------------------------------------------------
// Either display the help message or startup the application.
//--------------------------------------------------------------------------

// Start-up the script mods.
onStart();


